@@ -18,7 +18,6 @@
 /* Mathematical expression manipulations for WIMS */
 
 #include "mathexp.h"
-#include "../../Lib/libwims.h"
 
 char parmbuf[MAX_LINELEN+1];
 char *objline[MAX_OBJLINES];
@@ -26,8 +25,6 @@ int objlinecnt, thisobjline, thislinelen;
 int expl1, expl2;
 char *reqtype;
 int logdir, linelogdir;
-char *fnd_position;
-char *fnd_nextpos;
 
 regex regexchk[MAX_REGEX];
 
@@ -76,7 +73,6 @@ int main()
     char *p;
     int i;
     string_modify=string_modify1;
-    error1=error; error2=error; error3=error;
     p=getenv("wims_exec_parm");
     if(p==NULL || *p==0) error("No input data.");
     snprintf(parmbuf,sizeof(parmbuf),"%s",p);
