@@ -16,10 +16,13 @@
  */
 
 /* Routines treating user scores. */
+#include "wims.h"
 
 double oldfactor=0.85;     /* quality factor, should remain stable. */
 
-enum {sr_require, sr_weight, sr_score, sr_mean, sr_remain, sr_last, sr_try, sr_best, sr_level};
+enum {sr_require, sr_weight, sr_score, sr_mean, sr_remain,
+  sr_last, sr_try, sr_best, sr_level, sr_new, sr_seedlast, sr_seedscorelast,
+  sr_seedscores, sr_seedlastcnt};
 char scorebuf[MAX_CLASSEXOS*sizeof(scoreresult)+32];
 struct scoreresult *rscore;
 int scorecnt;
@@ -36,23 +39,29 @@ int score_statussheet=-1;
 int score_statusisexam=-1;
 double scorerecfactor=0.9;
 
-/* gather user score, core routine. */
+/* gather user score, core routine.
+  rscore data is created (table of structures).
+  according to the value of wims_sheet or wims_exo
+*/
 int getscoreuser(char *classe, char *user)
 {
-  int i, j, osh, sh;
-  double s;
+  int i, osh, sh;
   char *nowuser, *nowsheet, *nowexo, *nowscore;
   char *pp;
-  if(*user==0) {
+  if(user==NULL || *user==0) {
     user=getvar("wims_user");
     if(user==NULL || *user==0) return -1;
   }
   if(strcmp(classe,rscore_class)==0 && strcmp(user,rscore_user)==0) return 0;
   nowsheet=nowexo=nowscore="";
-  nowuser=getvar("wims_user"); if(nowuser!=NULL && strcmp(user,nowuser)==0) {
-    nowscore=getvar("module_score"); if(nowscore!=NULL && *nowscore!=0) {
-      nowsheet=getvar("wims_sheet"); if(nowsheet==NULL) nowsheet="";
-      nowexo=getvar("wims_exo"); if(nowexo==NULL) nowexo="";
+  nowuser=getvar("wims_user");
+  if(nowuser!=NULL && strcmp(user,nowuser)==0) {
+    nowscore=getvar("module_score");
+    if(nowscore!=NULL && *nowscore!=0) {
+      nowsheet=getvar("wims_sheet");
+      if(nowsheet==NULL) nowsheet="";
+      nowexo=getvar("wims_exo");
+      if(nowexo==NULL) nowexo="";
     }
     else nowscore="";
   }
@@ -67,27 +76,30 @@ int getscoreuser(char *classe, char *user)
     }
     else internal_error("getscoreuser(): communication error with wimslogd.");
   }
+/* 3 is here the length of "OK " */
   pp=scorebuf+sizeof(int)+3; rscore=(struct scoreresult *) pp;
   scorecnt=(i-sizeof(int)-3)/sizeof(scoreresult);
   if(scorecnt>MAX_CLASSEXOS) module_error("too_many_exercises");
-  s=0; for(i=osh=0;i<scorecnt;i++) {
-    sh=(rscore[i].num>>8)+1;
-    if(sh<1 || sh>MAX_SHEETS) break;
-    if(osh>0 && osh<sh) {scoresum[osh-1]=s; s=0;}
-    for(;osh<sh && osh<MAX_SHEETS;osh++) sheetstart[osh]=i;
-    s+=rscore[i].require*rscore[i].weight;
+  osh=-1;
+  for(i=totsheets=0;i<scorecnt;i++) {
+    sh=rscore[i].sh;
+    /* MAX_SHEETS doit tre infrieur  255 donc on n'atteint pas l'examen */
+    if (sh >= MAX_SHEETS) break;
+    while(osh<sh)
+      {++osh; scoresum[osh]=shexocnt[osh]=0; sheetstart[osh]=i;}
+    scoresum[sh]+=rscore[i].require*rscore[i].weight*rscore[i].active;
+    shexocnt[sh]++;
   }
-  if(osh>0) scoresum[osh-1]=s; totsheets=osh;
-  for(j=0;j<totsheets-1;j++) shexocnt[j]=sheetstart[j+1]-sheetstart[j];
-  shexocnt[totsheets-1]=i-sheetstart[totsheets-1];
-  examstart=i; examcnt=scorecnt-examstart;
+  totsheets=osh+1;
+
+  examstart=i;  examcnt=scorecnt-examstart;
   mystrncpy(rscore_class,classe,sizeof(rscore_class));
   mystrncpy(rscore_user,user,sizeof(rscore_user));
   return 0;
 }
 /* work : exo */
 char *scorepname[]={
-     "class","user","sheet","work","exam"
+  "class","user","sheet","work","exam"
 };
 #define scorepname_no (sizeof(scorepname)/sizeof(scorepname[0]))
 char score_class[MAX_CLASSLEN+1];
@@ -136,6 +148,8 @@ void _scoreparm(char *p)
      *pf=sav; ovlstrcpy(pn, pf);
     }
     *p=0;
+  /* the commands are OK from a non trusted module if the user and the
+  class are not precised, so it can be only the user in his class */
     if((*score_class!=0 || score_user[0]!=0) && !trusted_module()) {
      module_error("not_trusted"); return;
     }
@@ -159,39 +173,57 @@ void _scoreparm(char *p)
 
 void _getscore(char *p,int dtype)
 {
-    int i,sh,ex,osh;
-    float d;
-    char *p1;
+  int i,sh,ex,osh;
+  float d;
+  char dc[SEEDSIZE];
+  char ds[SEEDSCORES];
+  char *p1;
 
-    _scoreparm(p);
-    if(*score_class==0 || *score_user==0) return;
-    if(getscoreuser(score_class,score_user)<0) return;
-/*  2^8 no_sheet + no_exo_in_the_sheet so limit no_exo to 2^8 */
-    for(i=osh=0,p1=p;i<scorecnt && p1-p<MAX_LINELEN-32;i++) {
-     sh=(rscore[i].num>>8)+1; if(sh<1 || sh>MAX_SHEETS) break;
-     if(score_sheet!=0) {
-         if(sh<score_sheet) continue;
-         if(sh>score_sheet || sh>MAX_SHEETS) break;
-     }
-     ex=((rscore[i].num)&255)+1;
-     if(score_exo!=0 && ex!=score_exo) continue;
-     if(osh!=0 && sh!=osh) *p1++='\n';
-     switch(dtype) {
-         case sr_require: {d=rscore[i].require; break;}
-         case sr_weight: {d=rscore[i].weight; break;}
-         case sr_score: {d=rscore[i].score; break;}
-         case sr_mean: {d=rscore[i].mean; break;}
-         case sr_remain: {d=rscore[i].require-rscore[i].score; break;}
-         case sr_last: {d=rscore[i].last; break;}
-         case sr_try: {d=rscore[i].try; break;}
-         case sr_best: {d=rscore[i].best; break;}
-         case sr_level: {d=rscore[i].level ;break;}
-         default: {d=0; break;}
-     }
-     p1=moneyprint(p1,d); *p1++=' ';
-     osh=sh;
+  _scoreparm(p);
+  if(*score_class==0 || *score_user==0) return;
+  if(getscoreuser(score_class,score_user)<0) return;
+  for(i=osh=0,p1=p;i<scorecnt && p1-p<MAX_LINELEN-32;i++) {
+    sh=rscore[i].sh+1; if(sh<1 || sh>MAX_SHEETS) break;
+    if(score_sheet!=0) {
+      if(sh<score_sheet) continue;
+      if(sh>score_sheet || sh>MAX_SHEETS) break;
     }
-    *p1++='\n'; *p1=0;
+    ex=rscore[i].exo+1;
+    if(score_exo!=0 && ex!=score_exo) continue;
+    if(osh!=0 && sh!=osh) *p1++='\n';
+    switch(dtype) {
+      case sr_require: {d=rscore[i].require; break;}
+      case sr_weight: {d=rscore[i].weight; break;}
+      case sr_score: {d=rscore[i].score; break;}
+      case sr_mean: {d=rscore[i].mean; break;}
+      case sr_remain: {d=rscore[i].require-rscore[i].score; break;}
+      case sr_last: {d=rscore[i].last; break;}
+      case sr_try: {d=rscore[i].try; break;}
+      case sr_new: {d=rscore[i].new; break;}
+      case sr_best: {d=rscore[i].best; break;}
+      case sr_level: {d=rscore[i].level; break;}
+      case sr_seedscorelast: {d=rscore[i].seedscorelast; break;}
+      case sr_seedlastcnt: {d=rscore[i].seedlastcnt; break;}
+      case sr_seedlast: { mystrncpy(dc,rscore[i].seedlast,SEEDSIZE);
+        break;
+      }
+      case sr_seedscores: {
+        mystrncpy(ds,rscore[i].seedscores,SEEDSCORES); break;
+      }
+      default: {d=0; break;}
+    }
+    switch(dtype) {
+      case sr_seedlast:
+        { snprintf(p1,SEEDSIZE, "%s", dc); p1+=strlen(p1); break; }
+       case sr_seedscores:
+        { snprintf(p1,SEEDSCORES, "%s", ds); p1+=strlen(p1); break; }
+      default:
+        { p1=moneyprint(p1,d); }
+    }
+    *p1++=' ';
+    osh=sh;
+  }
+  *p1++='\n'; *p1=0;
 }
 
 /* gather user score. */
@@ -223,17 +255,21 @@ void calc_getscoreweight(char *p)
 {
   _getscore(p,sr_weight);
 }
-/* user last score*/
+/* user last score */
 void calc_getscorelast(char *p)
 {
   _getscore(p,sr_last);
 }
-
-/* gather user score try numbers. */
+/* gather user score try numbers with score activated. */
 void calc_getscoretry(char *p)
 {
   _getscore(p,sr_try);
 }
+/* gather all user try numbers (terminated or not, but with notation activated)*/
+void calc_getscorealltries(char *p)
+{
+  _getscore(p,sr_new);
+}
 
 /* if the required points are 10* N, gather N user best scores */
 void calc_getscorebest(char *p)
@@ -250,10 +286,10 @@ void calc_getscorelevel(char *p)
 /* percentage of work done for each sheet:
  * score mean best level
  * as follows
- * score=100*cumulative_points/required_points) (< 100)
+ * score=100*cumulative_points/required_points (< 100)
  * mean=quality (<10)
  * best=10*(required/10 best_scores)/required_points (< 100)
- * level=minimum of the required/10 best_scores (< 10)
+ * level=minimum of the required/10 best_scores (< 100)
  */
 
  void calc_getscorepercent(char *p)
@@ -273,40 +309,61 @@ void calc_getscorelevel(char *p)
     if(scoresum[i]<=0) *p1++='\n';
     tot=mean=totb=totl=totw=0; jend=sheetstart[i]+shexocnt[i];
     for(j=sheetstart[i];j<jend;j++) {
-/* if mean<1 then ignore score.
- * if mean<2 then half score.
- */
+      /* if mean<1 then ignore score.
+      * if mean<2 then half score.
+      */
       if(rscore[j].mean>=1) {
         double dt=rscore[j].score;
         float db=rscore[j].best;
         float dl=rscore[j].level;
         if(rscore[j].mean<2) {dt=dt/2; db=db/2; dl=dl/2;}
-        d=dt*rscore[j].weight;
-/* quality */
+        d=dt*rscore[j].weight*rscore[j].active;
+        /* quality; barycenter with coefficients d */
         mean+=rscore[j].mean*d;
-/* cumulative score */
+        /* cumulative score */
         tot+=d;
-/* best */
-        totb+=db*rscore[j].weight;
-/* level */
-        totl+=dl*rscore[j].weight;
-        totw+=rscore[j].weight;
+        /* best */
+        totb+=db*rscore[j].weight*rscore[j].active;
+        /* level */
+        totl+=dl*rscore[j].weight*rscore[j].active;
+        totw+=rscore[j].weight*rscore[j].active;
       }
     }
     if(tot>0) {d=mean/tot;} else d=0;
-/* cumulative score */
+    /* cumulative score */
     p1=moneyprint(p1,rint(100*tot/scoresum[i])); *p1++=' ';
-/* quality */
+    /* quality */
     p1=moneyprint(p1,d); *p1++=' ';
-/* best */
+    /* best */
     p1=moneyprint(p1,rint(100*totb/scoresum[i])); *p1++=' ';
-/* level */
+    /* level */
     p1=moneyprint(p1,rint(10*totl/totw));
 
    *p1++='\n';
   }
   *p1=0;
 }
+/* [seed1,score1;seed2,score2;...] at most MAX_SCORESEED */
+void calc_getseedscores(char *p)
+{
+  _getscore(p,sr_seedscores);
+}
+/* last seed (0 if there is no seed)*/
+void calc_getseedlast(char *p)
+{
+  _getscore(p,sr_seedlast);
+}
+/* last score (-1 if does not exist) */
+void calc_getseedscorelast(char *p)
+{
+  _getscore(p,sr_seedscorelast);
+}
+
+/* number of occurences of the last seed */
+void calc_getseedlastcnt(char *p)
+{
+  _getscore(p,sr_seedlastcnt);
+}
 
 /* Returns the status of a sheet, or -1 if error */
 int getsheetstatus(char *classe, int sheet)
@@ -325,10 +382,11 @@ int getsheetstatus(char *classe, int sheet)
   return i;
 }
 
-/* return 1 if a word of bf2 is a substring of host.
+/* return 1 if a word of bf2 is a substring of host
+  and if time restrictions are verified for wims_user
  * Content of bf2 is destroyed.
  */
-int _subword(char bf2[])
+int _subword(char bf2[],char *ftbuf)
 {
   char *p1, *p2;
   for(p1=strchr(bf2,'\\'); p1!=NULL; p1=strchr(p1+1,'\\')) {
@@ -342,26 +400,41 @@ int _subword(char bf2[])
     if(!isalnum(*(p1+1))) continue;
     p2=find_word_end(p1); if(p2>=p1+MAX_NAMELEN) continue;
     memmove(buf2, p1+1, p2-p1-1); buf2[p2-p1-1]=0;
-    snprintf(buf,sizeof(buf),"user__%s",buf2);
+    /* get value of technical variable */
+    snprintf(buf,sizeof(buf),"user_techvar_%s",buf2);
     if(strcmp(userp,"supervisor")==0)
       mkfname(fbuf,"%s/%s/supervisor",class_base,classp);
     else
       mkfname(fbuf,"%s/%s/.users/%s",class_base,classp2,userp);
-    getdef(fbuf,buf,buf2); if(buf2[0]==0) ovlstrcpy(buf2,"none");
-    string_modify(bf2,p1,p2,buf2);
+    getdef(fbuf,buf,buf2);
+    if(buf2[0]==0) ovlstrcpy(buf2,"EMPTY"); /* in case of no value defined for the user*/
+    /* get time restriction for this value */
+    snprintf(buf,sizeof(buf),"techvar_%s",buf2);
+    /* mkfname(fbuf,"%s/%s/.E%s",class_base,classp,sheet);
+     * read time restriction corresponding to the value of
+     * technical variable in the file of the sheet or exam
+    */
+    getdef(ftbuf,buf,buf2);
+    if(buf2[0]==0) ovlstrcpy(buf2,"none");
+    /*string_modify(bf2,p1,p2,buf2);*/
+    bf2=buf2;
     p1+=strlen(buf2);
   }
   if((isexam || score_isexam) && bf2[0]=='#') return 1;
   if(wordchr(bf2,"none")!=NULL) return 0;
   if(wordchr(bf2,"all")!=NULL) return 1;
   p1=find_word_start(bf2); if(*p1==0) return 1;
+  /* check host and time */
   return checkhostt(p1);
 }
 
-/* Returns 1 if score registration is open, 0 otherwise. */
+/* Returns 1 if score registration is open for the user
+ * in variable wims_user, 0 otherwise.
+ */
 int _getscorestatus(char *classe, int sheet)
 {
   char nbuf[MAX_LINELEN+1], gbuf[MAX_LINELEN+1];
+  char ftbuf[MAX_FNAME+1]="";
   char *es;
 
   if(classe==NULL || *classe==0 || sheet<=0) return 1;
@@ -371,25 +444,26 @@ int _getscorestatus(char *classe, int sheet)
     accessfile(nbuf,"r","%s/%s/.E%d",class_base,classe,sheet);
     if(nbuf[0]=='#') return 1;
   }
-/* Global restriction data */
+  /* Global restriction data */
   accessfile(nbuf,"r","%s/%s/.security",class_base,classe);
   if(nbuf[0]) {
     _getdef(nbuf,"allow",gbuf);
-    if(*find_word_start(gbuf)!=0 && _subword(gbuf)==0)
+    if(*find_word_start(gbuf)!=0 && _subword(gbuf,ftbuf)==0)
       return 0;
     _getdef(nbuf,"except",gbuf);
-    if(*find_word_start(gbuf)!=0 && _subword(gbuf)==1)
+    if(*find_word_start(gbuf)!=0 && _subword(gbuf,ftbuf)==1)
       return 0;
   }
-
-/* Sheet restriction data */
+  /* Sheet restriction data; take in account the technical variables */
   if(isexam || score_isexam) es="E"; else es="";
   accessfile(nbuf,"r","%s/%s/.%s%d",class_base,classe,es,sheet);
   if(*find_word_start(nbuf)==0) return 1;
-  return _subword(nbuf);
+  /* preparing score restriction file name :*/
+  mkfname(ftbuf,"%s/%s/.%s%d",class_base,classe,es,sheet);
+  return _subword(nbuf,ftbuf);
 }
 
-/* Returns 1 if score registration is open, 0 otherwise. */
+/* Returns 1 if score registration is open for wims_user, 0 otherwise.*/
 int getscorestatus(char *classe, int sheet)
 {
   if(score_status<0 || score_statussheet!=sheet
@@ -407,8 +481,9 @@ int getscorestatus(char *classe, int sheet)
   }
   if(isexam && score_status==0) {
     char *p;
-    p=getvar("wims_user"); if(p==NULL || strcmp(p,"supervisor")!=0)
-                 user_error("exam_closed");
+    p=getvar("wims_user");
+    if(p==NULL || strcmp(p,"supervisor")!=0)
+      user_error("exam_closed");
   }
   return score_status;
 }
@@ -425,6 +500,16 @@ void calc_getscorestatus(char *p)
   else ovlstrcpy(p,"no");
 }
 
+/* whether there are too much tries for the exo return yes or no */
+void calc_getscoremaxexotry(char *p)
+{
+  _scoreparm(p);
+  if(*score_class==0 || *score_user==0 || score_sheet==0 || score_exo==0) return;
+  if(gettrycheck(score_class, score_user, score_sheet, score_exo)==1)
+    ovlstrcpy(p,"yes");
+  else ovlstrcpy(p,"no");
+}
+
 double exam_scoredata[MAX_EXOS];
 
 /* get current exam score */
@@ -433,9 +518,8 @@ void exam_currscore(int esh)
   char *p, *bf, pb[MAX_FNAME+1];
   char *s, *p1, *p2, *e1, *e2;
   int i;
-
   for(i=0;i<MAX_EXOS;i++) exam_scoredata[i]=-1000;
-/* session_prefix is not yet defined here */
+  /* session_prefix is not yet defined here */
   s=getvar("wims_session"); if(s==NULL || *s==0) return;
   mystrncpy(pb,s,sizeof(pb));
   p=strchr(pb,'_'); if(p!=NULL) *p=0;
@@ -463,13 +547,22 @@ void calc_examscore(char *p)
 {
   char *p1;
   int i;
+  char *withoutip;
 
   _scoreparm(p); *p=0;
+  withoutip=getvar("wims_examscore_withoutip");
+
   if(*score_class==0 || *score_user==0) return;
   if(getscoreuser(score_class,score_user)<0) return;
   p1=p;
-  for(i=0; i<examcnt && p1-p<MAX_LINELEN-32; i++) {
-    p1=moneyprint(p1,rscore[examstart+i].score); *p1++=' ';
+  if(withoutip!=NULL && strcmp(withoutip,"yes")==0) {
+    for(i=0; i<examcnt && p1-p<MAX_LINELEN-32; i++) {
+    p1=moneyprint(p1,rscore[examstart+i].best); *p1++=' ';
+    }
+  } else {
+    for(i=0; i<examcnt && p1-p<MAX_LINELEN-32; i++) {
+      p1=moneyprint(p1,rscore[examstart+i].score); *p1++=' ';
+    }
   }
   *p1++='\n';
   for(i=0; i<examcnt && p1-p<MAX_LINELEN-32; i++) {
@@ -494,7 +587,8 @@ int _depcheck(char *ds, struct scoreresult *rs, int ecnt)
     *p2++=0; p2=find_word_start(p2);
     for(p3=p2; myisdigit(*p3); p3++);
     if(p3<=p2) break;
-    *p3++=0; perc=atoi(p2);
+    if(*p3) *p3++=0;
+    perc=atoi(p2);
     if(perc<=0 || perc>100) break;
     for(pp=p1; *pp; pp++) if(!myisdigit(*pp)) *pp=' ';
     tgot=ttot=tmean=0; sum=0;
@@ -540,7 +634,7 @@ int depcheck(char *sh, int exo, char *deps)
 
 int exam_depcheck(char *deps, int exam)
 {
-  struct scoreresult esc[MAX_EXOS];
+  static struct scoreresult esc[MAX_EXOS];
   int i;
   exam_currscore(exam);
   for(i=0;i<MAX_EXOS;i++) {
@@ -550,3 +644,58 @@ int exam_depcheck(char *deps, int exam)
   }
   return _depcheck(deps,esc,MAX_EXOS);
 }
+
+/* public sheet gives she=0 */
+/* return 1 if score is no more taken in account because of exotrymax */
+int gettrycheck(char *classe, char *user, int she, int exo) {
+  char *s;
+  int sh, ex, i;
+  if(classe==NULL || *classe==0 || user==NULL || *user==0) return 0;
+  if (she<=0) return 0;
+    if(strcmp(user,"supervisor")==0) return 0;
+  s=getvar("exotrymax");
+  if(s==NULL || *s==0) return 0;
+  getscoreuser(classe,user);
+  char *p=getvar("wims_scorereg");
+  for(i=0;i<scorecnt;i++) {
+    sh=rscore[i].sh+1;
+    if (she!=sh) continue;
+    ex=rscore[i].exo+1;
+    if(exo!=ex) continue;
+    if(cmd_type==cmd_new || cmd_type==cmd_renew)
+      if(rscore[i].new >= atoi(s)) {
+        if(strcmp(p,"suspend")!=0) setvar("wims_scorereg","exotrymax");
+        return 1;
+    }
+    if(rscore[i].new > atoi(s)){
+      if(strcmp(p,"suspend")!=0) setvar("wims_scorereg","exotrymax");
+      return 1;
+    }
+  }
+  return 0;
+}
+
+/* return seed if the seed should be kept and NULL if not */
+char *getseedscore(char *classe, char *user, int she, int exo) {
+  char *s;
+  int sh, ex, i;
+  if (she<=0) return NULL;
+  if(strcmp(user,"supervisor")==0) return NULL;
+  s=getvar("seedrepeat");
+  if(s==NULL || *s==0 || atoi(s)<=0) return NULL;
+  getscoreuser(classe,user);
+  for(i=0;i<scorecnt;i++) {
+    sh=rscore[i].sh+1;
+    if (she!=sh) continue;
+    ex=rscore[i].exo+1;
+    if(exo!=ex) continue;
+    if(atoi(rscore[i].seedlast)==0) {
+      char *seed=getvar("wims_seed");
+      return seed;
+    }
+    if(rscore[i].seedlastcnt > MAX_SEEDSCORE) return NULL;
+    if(atoi(s)>0 && rscore[i].seedlastcnt >= atoi(s) && cmd_type!=cmd_next) return NULL;
+    if(rscore[i].seedscorelast < 10) return rscore[i].seedlast;
+  };
+  return NULL;
+}
